#ifndef HOSVD_H_INCLUDED
#define HOSVD_H_INCLUDED

#include <vector>
#include <utility>
#include <functional>
#include <chrono>
#include <limits>

#include <Eigen/Dense>
#include <Eigen/SVD>
#include <Eigen/Sparse>
#include <unsupported/Eigen/KroneckerProduct>

#include "utilities2.h"
#ifdef _OPENMP
#include "omp.h"
#endif

#define YUV       0
#define YCBCR     1
#define OPP       2
#define RGB       3
#define NONE      4

typedef struct {
    bool hasOutputNoisy, hasOutputDenoised1, hasOutputDenoised2, hasOutputOracle;
    char *outputNoisy, *outputDenoised1, *outputDenoised2, *outputOracle;
} Output_params;

typedef struct {
    unsigned int kHard;
    unsigned int K;
    unsigned int nHard;
    float thresholdBM;
    float thresholdHD;
    unsigned int pHard;
    unsigned int nbThreads;
} Options_params;

typedef struct {
    bool hasInput, hasSigma, hasColorspace;
    char *input;
    unsigned int colorspace;
    float sigma;
    Output_params out;
    Options_params params;
} Params;


/** ----------------------- **/
/** - Main data structure - **/
/** ----------------------- **/
template<typename T1, typename T2> struct greaters_similarity{
    bool operator() (const std::pair<T1,T2>& E1, const std::pair<T1,T2>& E2 ) {
        return (E1.first < E2.first);
    }
};

/** ------------------ **/
/** - Main functions - **/
/** ------------------ **/
//! Main function
int hosvd(
    const float sigma
,   std::vector<float> &img_clean
,   std::vector<float> &img_noisy
,   std::vector<float> &img_denoised
,   std::vector<float> &img_denoised_wiener
,   std::vector<float> &img_denoised_oracle
,   const unsigned width
,   const unsigned height
,   const unsigned chnls
,   const unsigned color_space
,   Output_params &out
,   Options_params &settings
);

/** ---------------------------------- **/
/** -             HOSVD              - **/
/** ---------------------------------- **/
void hosvd_sub(const float sigma
, const std::vector<float> &img_noisy
, std::vector<float> &img_denoised
, const bool doWiener, const std::vector<float> &img_wiener
, const bool doOracle, const std::vector<float> &img_oracle
, const unsigned width, const unsigned height, const unsigned chnls
, const unsigned color_space
, const unsigned P, const unsigned Q
, const unsigned K, const unsigned pHW
, const unsigned p, const unsigned q
, const float thresholdHD
, const float thresholdBM
, unsigned nb_threads);

/** ---------------------------------- **/
/** - Preprocessing / Postprocessing - **/
/** ---------------------------------- **/
void insert_boundedSet (const std::pair<float, unsigned int> &value,
                        std::vector<std::pair<float, unsigned int> > &set,
                        const unsigned int K);

float compute_dist(std::vector< float>::const_iterator P1,
                   std::vector< float>::const_iterator P2,
                   const unsigned int p, const unsigned int q,
                   const unsigned width, const unsigned height, const unsigned chnls,
                   const unsigned P, const unsigned Q);

void precompute_BM(std::vector<std::vector<std::pair<float, unsigned int> > > &patch_table // output table for each patch
        ,   const std::vector<float> &img, const unsigned width, const unsigned height, const unsigned chnls // input symetrized image
        ,   const unsigned p, const unsigned q
        ,   const unsigned K  // Max number of elements in output list for each input patch
        ,   const unsigned P, const unsigned Q  // Size PxQ for the proximity of patches
        ,   const float    thresholdBM // Threshold value to keep or not a similar patch
        ,   const std::vector<float> &sigma_table
        ,   unsigned nb_threads // number of OPENMP threads
        );
#endif // HOSVD_H_INCLUDED
